import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Enemy here.
 * 
 * @author Alvin Chong 
 * @version (a version number or a date)
 */
public class Enemy extends Actor
{
    private Player player;
    private Player target;
    private MyWorld m;
    private Counter counter;
    //declare instance variables
    private int chance = 0;
    int health = 1;
    int score = 0;
    boolean checkDeath = false;
    //Declare arrays of GreenfootImages for animation
    private GreenfootImage[] bounceRight;
    private GreenfootImage[] bounceLeft;

    //declare GreenfootImages for Standing
    private GreenfootImage slimeStandRight;
    private GreenfootImage slimeStandLeft;

    //Booleans for controlling movement
    private boolean bouncing;
    //Integers to control the speed of movement and animation
    private int slimeAnimationCounter;
    private int slimeAnimationDelay;
    private int slimeAnimationDelayCounter;

    public Enemy (Player p)
    {
        player = p;
        //Three Strings to create a file name for the animation
        String slimeFileName;
        String slimeFileNamePrefix = "Slime";
        String slimeFileNameSuffix = ".png";

        //Initialize arrays
        bounceRight = new GreenfootImage[7];
        bounceLeft = new GreenfootImage[7];

        //use a loop to populate image array
        for (int i = 0; i < bounceRight.length; i++)
        {
            //Create the file name
            slimeFileName = slimeFileNamePrefix + (i+1) + slimeFileNameSuffix;
            //use the file name for the sprite
            bounceRight[i] = new GreenfootImage(slimeFileName);
            //Create the left facing sprite by flipping it
            bounceLeft[i] = new GreenfootImage (bounceRight[i]);
            bounceLeft[i].mirrorHorizontally();
        }
        //import standing images
        slimeStandRight = new GreenfootImage ("Slime1.png");
        slimeStandLeft = new GreenfootImage (slimeStandRight);
        slimeStandLeft.mirrorHorizontally();

        //set starting values for control variables
        bouncing = false;
        // Amount of acts used as delay for changing frames 
        slimeAnimationDelay = 7;
        //start delay counter at 0
        slimeAnimationDelayCounter = 0;

        slimeAnimationCounter = 0;
        this.setImage (slimeStandRight);
    }

    /**
     * Act - do whatever the Enemy wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        // Add your action code here.
        slimeAnimationDelayCounter++;
        if(slimeAnimationDelayCounter == slimeAnimationDelay)
        {
            slimeAnimationCounter++;
            slimeAnimationDelayCounter = 0;
        }

        if (slimeAnimationCounter > bounceRight.length-1)
        {
            slimeAnimationCounter = 0;
        }
        followPlayer();
        removeEnemy();
    } 

    /**
     * Pursues the player,changes direction when needed
     */
    public void followPlayer ()
    {
        //infinite loop, 100% chance that the enemy will follow the player
        if (chance != 1)    
        {    
            //if the Player's X coordinate is greater than the enemy's X coordinate,
            //then the enemy will move to the right
            if (player.getX() > getX())
            {
                moveRight();
            }
            //If the Player's X coordinate is less than the enemy's X coordinate,
            //then the enemy will move to the left
            if (player.getX() < getX())
            {
                moveLeft();
            }

        }  
    }

    /**
     * Moves the Enemy to the right
     */
    public void moveRight()
    {
        //X coordinate is changed by a factor of 2
        int x = (getX() + 2);  
        //no change in Y coordinate due to horizontal movement
        int y = (getY());  
        //change the location of the enemy
        setLocation(x, y); 
        //Change the image of the enemy to the according images
        setImage (bounceRight [slimeAnimationCounter]);
    }

    /**
     * Moves the Enemy to the left
     */
    public void moveLeft()
    {
        //X coordinate is changed by a factor of 2
        int x = (getX() - 2); 
        //no change in Y coordinate due to horizontal movement
        int y = (getY());  
        //change the location of the enemy
        setLocation(x, y);  
        //Change the image of the enemy to the according images
        setImage (bounceLeft [slimeAnimationCounter]);
    }

    /**
     * Reduces the amount of health on the enemy
     */
    public void damage(){
        //decrease the health by 1
        health--;
    }
    
    /**
     * Calls the world
     * @param world sets the world as it's target
     */
    protected void addedToWorld (World world)
    {
        m=(MyWorld) world;  
    }

    /**
     * Removes the enemy from the world when dead
     */
    public void removeEnemy()
    {
        //if the health is 0 then checkDeath = true
        if (health == 0)
        {
            checkDeath = true;
        }
        //if checkDeath is true then add a point to the scorebar and remove the object
        if (checkDeath)
        {
            ((Counter)getWorld().getObjects(Counter.class).get(0)).changeCount(1);  
            m.removeObject(this);
        }
        //subtract the amount of enemies from spawnCounter in the mainworld
        m.spawnCounter--;
    }
}
