import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Description:
 * main class of the program, the player's controls are tracked in this class.
 * Background information such as music is also controlled in this class
 * 
 * [][][][][][][][][]Credits[][][][][][][][][]
 * 
 * Images ===================================
 * [edited using Adobe Photoshop CS3]
 * Scott Pilgrim by Bryan O'Malley
 * sprites from http://www.spriters-resource.com/other_systems/scottpilgrimvstheworld/sheet/33605/
 * 
 * Slimes by Nexon (Maplestory)
 * sprites from http://www.spriters-resource.com/pc_computer/maplestory/sheet/21657/
 * Music ====================================
 *
 *Super Smash Brothers Brawl (Battlefield Theme)
 *From Nintendo
 *      
 *      
 * Instructions =============================
 * Objective: Survive multiple waves of enemies, enemies will get progressively harder after each wave
 * 
 * Controls:
 * Left Key to move Left
 * Right Key to move Right
 * "Z" to attack Left
 * "X" to attack Right
 *
 * Known Issues and Bugs
 * -Performance Drop when attacking
 * -Visual Bug with the healthbar, when the bar reaches 0 the health is not at 0
 * 
 * @author  Alvin Chong
 * @version V.1 (June 6th 2013)
 * added basic character movement and background music
 * 
 * @version V.2 (June 8th 2013)
 * added enemies
 * 
 * @version V.3 (June 12th 2013)
 * added Collision detection
 * 
 * @version V.4 (June 14th 2013)
 * added scorekeeping
 * 
 * @version Final Submission Date
 * June 16th 2013
 */
public class MyWorld extends World
{
    //intialize variables and objects 
    private boolean isKeyPressed;
    private int maxSpawn = 3;
    int spawnCounter = 0;
    private int score = 0;
    private boolean checkMoving;
    private boolean checkEnemyDeath = false;
    private Player player;
    private HealthBar hpBar;
    private Enemy enemy;
    private Counter counter;
    private MainScreen main;

    /**
     * Constructor for objects of class MyWorld.
     * 
     */
    public MyWorld()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(600, 400, 1); 
        //calls the constructor within the player class
        player = new Player ();
        //calls the constructor withing the enemy class
        enemy = new Enemy (player);
        //calls the constructor within the HealthBar class
        hpBar = new HealthBar (100);
        //calls the constructor within the Counter class
        counter = new Counter ();
        //adds the player onto the coordinates (300,370)
        addObject (player, 300, 370);
        //adds the Health Bar onto the coordinates (125,70)
        addObject (hpBar, 125, 50);
        //adds the score counter onto the coordinates ()
        addObject (counter,500, 50);
        player.initHpBar (hpBar);
    }

    /**
     * Called on every act
     * -Calls the checkKeys method; used for movement
     * -Calls the playMusic method; used to play sounds, mainly the background music
     */
    public void act ()
    {
        checkKeys();

        //Generates a random number, if the number is 1 and the spawn counter is less than the maxSpawn then call the spawnEnemy method
        if (Greenfoot.getRandomNumber(50) == 1 && spawnCounter < maxSpawn)
        {
            spawnEnemy();
        }

    }

    /**
     * Procedure for checking which keys are pressed and moves the player accordingly
     */
    private void checkKeys()
    {
        // boolean to check if either direction was pressed
        isKeyPressed = false;
        if (Greenfoot.isKeyDown("right"))
        {
            player.walkRight();
            isKeyPressed = true;
            checkMoving = true;
        }
        if (Greenfoot.isKeyDown("left"))
        {
            player.walkLeft();
            isKeyPressed = true;
            checkMoving = true;
        }
        // In the event that no button is pressed, call the stopWalking method
        if (!(isKeyPressed))
        {
            player.stopWalking();
            checkMoving = false;
        }

        //creates a string that tracks the most recent key that is pressed
        String s=Greenfoot.getKey();
        if (s!=null)
        { 
            //if the x button is pressed,then call the attackingRight method in the player class
            if (s.equals("x") && checkMoving == false)
            {
                player.attackingRight();
                Greenfoot.delay(3);
            }
            //if the z button is pressed, then call the attackingLeft method in the player class
            if (s.equals("z") && checkMoving ==false)
            {
                player.attackingLeft();
                Greenfoot.delay(3);
            }

        }
    }
   
    /**
     * Spawns the enemy onto the world
     */
    private void spawnEnemy()
    {   
        int positionX;
        //increase the spawnCounter by 1
        spawnCounter++;
        if (Greenfoot.getRandomNumber(3) == 1){ //There is a 50% chance that the enemy is spawned on the right side
            positionX = getWidth() - 10 - Greenfoot.getRandomNumber(25);
        } else {    //There is a 50% chance that the enemy is spawned on the left side
            positionX = 10 + Greenfoot.getRandomNumber(25);
        }

        //Adds the enemy onto the coordinates 
        addObject( new Enemy(player), positionX, 370);
    }

    /**
     * Updates the score based on the amount of kills
     */
    public int updateScore()
    {
        //if the enemy dies then, increase the score and change the score
        if (enemy.checkDeath == true)
        {
            score++;
            counter.changeCount(score);
        }
        return score;

    }
}

