import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Player here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Player extends Actor
{
    //intialize instance variables
    private int currentHP=100;
    private int damage = 1;
    private boolean checkDeath = false;

    private HealthBar hpBar;
    // Declare arrays of GreenfootImages for animation
    private GreenfootImage[] walkingRight;
    private GreenfootImage[] walkingLeft;
    private GreenfootImage[] attackRight;
    private GreenfootImage[] attackLeft;
    // Declare GreenfootImages for standing
    private GreenfootImage standingRight;
    private GreenfootImage standingLeft;

    // Two booleans for controlling the flow of the action
    private boolean walking;
    private boolean facingRight;

    // Integers for controlling the speed of the movement and animation
    private int animationCounter;
    private int animationDelay;
    private int animationDelayCounter;
    private int speed;
    private int attackAnimationDelay;
    private int attackAnimationDelayCounter;
    private int attackAnimationCounter;

    /**
     * Constructor for P
     *layer imports the necessary images, sets
     * values for variables and sets this Actor's image to begin.
     */
    public Player ()
    {
        // Three Strings together to build file name for the Walk animation
        String fileName;
        String fileNamePrefix = "Walk";
        String fileNameSuffix = ".png";

        //Three Strings togther to build file name for the attack animation
        String attackFileName;
        String attackNamePrefix = "Attack";
        String attackNameSuffix = ".png";

        // Initialize arrays
        walkingRight = new GreenfootImage[8];
        walkingLeft = new GreenfootImage[8];
        attackRight = new GreenfootImage[6];
        attackLeft = new GreenfootImage[6];

        // Use a loop to populate the two image arrays
        for (int i = 0; i < walkingRight.length; i++)
        {
            // Build the file name: Eg. right02.png
            fileName = fileNamePrefix + (i+1) + fileNameSuffix;
            // Use the file for the right facing sprite
            walkingRight[i] = new GreenfootImage(fileName);
            // Build the left facing sprite by flipping the
            // right facing sprite
            walkingLeft[i] = new GreenfootImage (walkingRight[i]);
            walkingLeft[i].mirrorHorizontally();
        }

        for (int z =0; z < attackRight.length; z++)
        {
            attackFileName = attackNamePrefix + (z+1) + attackNameSuffix;

            attackRight[z] = new GreenfootImage(attackFileName);

            attackLeft[z]= new GreenfootImage (attackRight[z]);
            attackLeft[z].mirrorHorizontally();
        }
        // import the right standing image
        standingRight = new GreenfootImage ("Idle.png");
        // create the left standing image from the right standing image
        standingLeft = new GreenfootImage (standingRight);
        standingLeft.mirrorHorizontally(); 

        // Set starting values for control variables
        walking = false;
        facingRight = true;
        // How fast the Actor will move
        speed = 3;
        // How many acts as a delay between changing frames
        animationDelay = 8;
        attackAnimationDelay = 6;
        // Start delay counter  at 0
        animationDelayCounter = 0;
        attackAnimationDelayCounter = 0;
        // Start the animation counter at 0. The animation counter keeps
        // track of what frame the animation is on, and gets reset if the
        // player changes direction. It is only updated every 
        // *animationDelayCounter* acts, so the animation isn't too fast.
        animationCounter = 0;
        attackAnimationCounter = 0;
        // Set starting image
        this.setImage (standingRight);

    }

    public void act() 
    {
        // animationDelayCounter is used to avoid the animation happening
        // too fast. Each act it increases by 1 until it hits *animationDelay*.
        animationDelayCounter++;
        if(animationDelayCounter == animationDelay)
        {
            animationCounter++;
            animationDelayCounter = 0;
        }

        attackAnimationCounter++;
        if(attackAnimationCounter == attackAnimationDelay)
        {
            attackAnimationCounter++;
            attackAnimationCounter = 0;
        }

        // Animation counter controls which frame is currently being displayed.
        // If it gets beyond the size of our array of images, reset it to zero
        if (animationCounter > walkingRight.length -1)
            animationCounter = 0;

        if (attackAnimationCounter > attackRight.length-1)
            attackAnimationCounter = 0;
        checkHit();
    }    

    /**
     * Called by WalkingWorld, causes the player to move left.
     */
    public void walkLeft()
    {
        // Check if direction is changing, and if so, reset counter
        if (facingRight)
            animationCounter = 0;
        // Set control booleans to not facing right and walking
        walking = true;
        facingRight = false;
        // Set the appropriate image. Note that animationCounter is
        // controlled by the act() method.
        setImage (walkingLeft[animationCounter]);
        // Move the actor
        setLocation (getX() - speed, getY());   

    }

    /**
     * Called by WalkingWorld, causes the player to move right.
     */
    public void walkRight()
    {
        // Check if direction is changing, and if so, reset counter
        if (!(facingRight))
            animationCounter = 0;
        // Set control booleans to facing right and walking
        walking = true;
        facingRight = true;
        // Set the appropriate image. Note that animationCounter is
        // controlled by the act() method.
        setImage (walkingRight[animationCounter]);
        // Move the actor
        setLocation (getX() + speed, getY());

    }

    public void attackingRight()
    {
        //if the actor is facing left then the attack animation counter is set to 0
        if (!(facingRight))
        {
            attackAnimationCounter = 0;
        }
        //set walking to false, so player cannot attack while moving
        walking = false;
        //set facingRight to true
        facingRight = true;
        //set the appropriate images
        setImage (attackRight [attackAnimationCounter]);
        //Check if the player is colliding with an instance of an enemy
        //Code from James Lu
        Actor tempActor = getOneIntersectingObject (Enemy.class);
        if (tempActor instanceof Enemy){
            Enemy e = (Enemy) tempActor;
            e.damage();
        }
    }

    public void attackingLeft()
    {
        //if the actor is facing right then the attack animation counter is set to 0
        if (facingRight)
        {
            animationCounter=0;
        }
        //set walking to false, so player cannot attack while moving
        walking = false;
         //set facingRight to false
        facingRight = false;
        //set the appropriate images
        setImage(attackLeft [attackAnimationCounter]);
        //Check if the player is colliding with an instance of an enemy
        //Code from James Lu
        Actor tempActor = getOneIntersectingObject (Enemy.class);
        if (tempActor instanceof Enemy){
            Enemy e = (Enemy) tempActor;
            e.damage();
        }

    }

    /**
     * Called by MyWorld, causes the player to stand still.
     * This method gets called when the World decides that neither
     * directional button has been pushed.
     */
    public void stopWalking()
    {
        // Stop walking
        walking = false;
        // Reset animation counter
        animationCounter = 0;
        // Reset animation delay caounter
        animationDelayCounter = 0;
        // Set appropriate image based on which way player was most recently
        // moving
        if (facingRight)
            this.setImage (standingRight);
        else
            this.setImage (standingLeft);
    }    

    /**
     * Checks if the enemy is touching the player
     */
    private int checkHit()
    {
        //Check collision using the centre of the enemy image with instances
        //code from James Lu
        Actor tempActor = getOneObjectAtOffset (getImage().getWidth()/2,getImage().getHeight()/2,Enemy.class);
        //if the enemy is touching the player then reduce HP and update the image from the HealthBar class
        if (tempActor instanceof Enemy){
            Enemy e = (Enemy) tempActor;
            currentHP -= damage;
            hpBar.changeHP(currentHP);
        }
        return currentHP;

    }
    
    /**
     * The value of the HP in this class is taken on by the HealthBar class
     */
    public void initHpBar (HealthBar hpBar)
    {
        this.hpBar = hpBar;
    }


}